@@ -1,9 +1,9 @@
-#!/usr/bin/python
+#!/usr/bin/python3
 
 '''Xorg Apport interface
 
-Copyright (C) 2007, 2008 Canonical Ltd.
-Author: Bryce Harrington <bryce.harrington@ubuntu.com>
+Copyright (C) 2007-2012 Canonical Ltd.
+Author: Bryce Harrington <bryce@canonical.com>
 
 This program is free software; you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -14,12 +14,18 @@ the full text of the license.
 Testing:  APPORT_STAGING="yes"
 '''
 
+from __future__ import absolute_import, print_function, unicode_literals
+
 import os.path
 import glob
 import re
 import subprocess
 from apport.hookutils import *
-from launchpadlib.launchpad import Launchpad
+
+try:
+    from launchpadlib.launchpad import Launchpad
+except ImportError:
+    pass
 
 core_x_packages = [
     'xorg', 'xorg-server', 'xserver-xorg-core', 'mesa'
@@ -27,9 +33,6 @@ core_x_packages = [
 video_packages = [
     'xserver-xorg-video-intel', 'xserver-xorg-video-nouveau', 'xserver-xorg-video-ati',
     ]
-keyboard_packages = [
-    'xorg', 'xkb-data', 'xkeyboard-config', 'xserver-xorg-input-keyboard', 'xserver-xorg-input-evdev'
-    ]
 opt_debug = False
 
 ######
@@ -41,6 +44,13 @@ def debug(text):
     if opt_debug:
         sys.stderr.write("%s\n" %(text))
 
+def is_number(s):
+    try:
+        float(s)
+        return True
+    except ValueError:
+        return False
+
 def retrieve_ubuntu_release_statuses():
     '''
     Attempts to access launchpad to get a mapping of Ubuntu releases to status.
@@ -75,9 +85,15 @@ def is_process_running(proc):
         return False
     return True
 
-def is_xorg_input_package(pkg):
+def is_xorg_keyboard_package(pkg):
     if (pkg == 'xkeyboard-config' or
-        pkg == 'xkb-data' or
+        pkg == 'xkb-data'):
+        return True
+    else:
+        return False
+
+def is_xorg_input_package(pkg):
+    if (is_xorg_keyboard_package(pkg) or
         pkg[:18] == 'xserver-xorg-input' or
         pkg[:10] == 'xf86-input'):
         return True
@@ -208,18 +224,22 @@ def check_is_supported(report, ui=None):
         report['UnreportableReason'] = 'Unsupported Ubuntu Release'
         return False
 
-    elif status == "Supported" or status == "Current Stable Release":
+    elif ((status == "Supported" or status == "Current Stable Release") and
+          report.get('ProblemType', '') != 'Crash'):
         # TODO: Perhaps avoid this check if the time since the release < 1 month?
+        # lsb_release -rs gives 12.04
+        # int(release.version.split('.')[1])+2
         response = ui.choice(
-            "Development is completed for the '%s' version of Ubuntu, so\n"
-            "you should probably use technical support channels unless you know for\n"
-            "certain it should be reported here?" %(distro_codename),
+            "'%s' is no longer under development, but technical support is still \n"
+            "available and will give you quicker results than filing a bug here.  \n"
+            "Also, if you do have a bug, we will give it higher priority if you've gone \n"
+            "through technical support channels first." %(distro_codename),
             [
-                "I don't know",
-                "Yes, I already know the fix for this problem.",
-                "Yes, The problem began right after doing a system software update.",
-                "Yes.",
-                "No, please point me to a good place to get support.",
+                "I don't know what to do",
+                "Continue. I already know of a patch to fix this problem.",
+                "Continue. Yes, I have discussed this with technical support.",
+                "Continue. I don't need technical support.",
+                "Please point me to where I can get technical support.",
                 ]
             )
         if response == None:
@@ -231,30 +251,22 @@ def check_is_supported(report, ui=None):
             ui.information("Thanks for helping improve Ubuntu!  Tip:  If you attach the fix to the bug report as a patch, it will be flagged to the developers and should get a swifter response.")
             return True
 
-        # Regression after a system update
-        elif 2 in response:
-            report['Tags'] += ' ' + 'regression-update'
-            response = ui.yesno("Thanks for reporting this regression in Ubuntu %s.  Do you know exactly which package and version caused the regression?" %(distro_codename))
-            if response:
-                ui.information("Excellent.  Please make sure to list the package name and version in the bug report's description.  That is vital information for pinpointing the cause of the regression, which will make solving this bug go much faster.")
-                report['Tags'] += ' ' + 'needs-reassignment'
-                return True
-            else:
-                ui.information("Alright, please indicate roughly when you first started noticing the problem.  This information will help in narrowing down the suspect package updates.")
-                return True
-
         # Referred by technical support
-        elif 3 in response:
+        elif 2 in response:
+            report['Tags'] += ' ' + 'referred-by-support'
             ui.information("Thanks for using technical support channels before filing this report.  In your bug report, please restate the issue and include a link or transcript of your discussion with them.")
             return True
 
+        # Just want to file a bug report
+        elif 3 in response:
+            return True
+
         # Anything else should be redirected to technical support channels
         else:
-            ui.information("http://askubuntu.com is the best place to get free help with technical issues.\n\n"
-                           "See http://www.ubuntu.com/support for paid support and other free options.")
+            ui.information("http://www.ubuntu.com/support/ is the best place to get free help with technical issues.")
             report['UnreportableReason'] = 'Please work this issue through technical support channels first.'
             return False
- 
+
     return True
 
 def check_is_reportable(report, ui=None):
@@ -278,14 +290,7 @@ def check_is_reportable(report, ui=None):
 
     if os.path.exists('/var/log/nvidia-installer.log'):
         # User has installed nVidia drivers manually at some point.
-        # This is likely to have caused problems.
-        if ui and not ui.yesno("""It appears you may have installed the nVidia drivers manually from nvidia.com at some point in the past.  This can cause problems with the Ubuntu-supplied drivers.
-
-If you have not already uninstalled the drivers downloaded from nvidia.com, please uninstall them and reinstall the Ubuntu packages before filing a bug with Ubuntu.  If you have uninstalled them, then you may want to remove the file /var/log/nvidia-installer.log as well.
-
-Have you uninstalled the drivers from nvidia.com?"""):
-            report['UnreportableReason'] = 'The drivers from nvidia.com are not supported by Ubuntu.  Please uninstall them and test whether your problem still occurs.'
-            return
+        # This can cause problems, although the packaging scripts should prevent this situation now.
         attach_file(report, '/var/log/nvidia-installer.log', 'nvidia-installer.log')
         report['Tags'] += ' ' + 'possible-manual-nvidia-install'
 
@@ -315,11 +320,10 @@ def attach_dist_upgrade_status(report, ui=None):
     release of ubuntu, or was a fresh install of this release.
     '''
     attach_file_if_exists(report, "/var/log/dpkg.log", "DpkgLog")
-    if os.path.lexists('/var/log/dist-upgrade/apt.log'):
-        # TODO: Not sure if this is quite exactly what I want, but close...
+    if os.path.lexists('/var/log/dist-upgrade/main.log'):
         attach_command_output(
             report,
-            ['head', '-n', '1', '/var/log/dist-upgrade/apt.log'],
+            ['tail', '-n', '1', '/var/log/dist-upgrade/main.log'],
             'DistUpgraded')
         return True
     else:
@@ -355,7 +359,7 @@ def attach_xorg_package_versions(report, ui=None):
     Gathers versions for various X packages of interest
     '''
     for package in [
-        "xserver-xorg",
+        "xserver-xorg-core",
         "libgl1-mesa-glx",
         "libgl1-mesa-dri",
         "libgl1-mesa-dri-experimental",
@@ -366,12 +370,12 @@ def attach_xorg_package_versions(report, ui=None):
         "xserver-xorg-video-ati",
         "xserver-xorg-video-nouveau"]:
         report['version.%s' %(package)] = package_versions(package)
-    if 'Architecture' in report and report['Architecture'] == 'amd64':
+    if report.get('Architecture', '') == 'amd64':
         report['version.ia32-libs'] = package_versions('ia32-libs')
 
-def attach_2d_info(report, ui=None):
+def attach_xorg_info(report, ui=None):
     '''
-    Attaches various data for debugging basic graphics issues.
+    Attaches basic xorg debugging info
     '''
     attach_file_if_exists(report, '/var/log/boot.log', 'BootLog')
     attach_file_if_exists(report, '/var/log/plymouth-debug.log', 'PlymouthDebug')
@@ -379,6 +383,21 @@ def attach_2d_info(report, ui=None):
     attach_file_if_exists(report, '/var/log/Xorg.0.log', 'XorgLog')
     attach_file_if_exists(report, '/var/log/Xorg.0.log.old', 'XorgLogOld')
 
+    if ui:
+        display_manager_files = {}
+        if os.path.lexists('/var/log/lightdm'):
+            display_manager_files['LightdmLog'] = 'cat /var/log/lightdm/lightdm.log'
+            display_manager_files['LightdmDisplayLog'] = 'cat /var/log/lightdm/x-0.log'
+            display_manager_files['LightdmGreeterLog'] = 'cat /var/log/lightdm/x-0-greeter.log'
+            display_manager_files['LightdmGreeterLogOld'] = 'cat /var/log/lightdm/x-0-greeter.log.old'
+
+        if ui.yesno("Your display manager log files may help developers diagnose the bug, but may contain sensitive information such as your hostname.  Do you want to include these logs in your bug report?") == True:
+            attach_root_command_outputs(report, display_manager_files)
+
+def attach_2d_info(report, ui=None):
+    '''
+    Attaches various data for debugging modesetting and graphical issues.
+    '''
     if os.environ.get('DISPLAY'):
         # For resolution/multi-head bugs
         attach_command_output(report, ['xrandr', '--verbose'], 'Xrandr')
@@ -392,31 +411,18 @@ def attach_2d_info(report, ui=None):
         # For font dpi bugs
         attach_command_output(report, ['xdpyinfo'], 'xdpyinfo')
 
-    if ui:
-        display_manager_files = {}
-        if os.path.lexists('/var/log/gdm'):
-            display_manager_files['GdmLog'] = 'cat /var/log/gdm/:0.log'
-            display_manager_files['GdmLog1'] = 'cat /var/log/gdm/:0.log.1'
-            display_manager_files['GdmLog2'] = 'cat /var/log/gdm/:0.log.2'
-
-        if os.path.lexists('/var/log/lightdm'):
-            display_manager_files['LightdmLog'] = 'cat /var/log/lightdm/lightdm.log'
-            display_manager_files['LightdmDisplayLog'] = 'cat /var/log/lightdm/:0.log'
-            display_manager_files['LightdmGreeterLog'] = 'cat /var/log/lightdm/:0-greeter.log'
-
-        if ui.yesno("Your display manager log files may help developers diagnose the bug, but may contain sensitive information such as your hostname.  Do you want to include these logs in your bug report?") == True:
-            attach_root_command_outputs(report, display_manager_files)
-
 def attach_3d_info(report, ui=None):
     # How are the alternatives set?
     attach_command_output(report, ['ls','-l','/etc/alternatives/gl_conf'], 'GlAlternative')
 
     # Detect software rasterizer
     xorglog = report.get('XorgLog', '')
+    if type(xorglog) is str:
+        xorglog = xorglog.encode('utf-8')
     if len(xorglog)>0:
-        if 'reverting to software rendering' in xorglog:
+        if b'reverting to software rendering' in xorglog:
             report['Renderer'] = 'Software'
-        elif 'Direct rendering disabled' in xorglog:
+        elif b'Direct rendering disabled' in xorglog:
             report['Renderer'] = 'Software'
 
     if ui and report.get('Renderer', '') == 'Software':
@@ -433,7 +439,7 @@ def attach_3d_info(report, ui=None):
         'GconfCompiz')
 
     # Compiz internal state if compiz crashed
-    if report.get('SourcePackage','Unknown') == "compiz" and report.has_key("ProcStatus"):
+    if report.get('SourcePackage','Unknown') == "compiz" and "ProcStatus" in report:
         compiz_pid = 0
         pid_line = re.search("Pid:\t(.*)\n", report["ProcStatus"])
         if pid_line:
@@ -470,8 +476,9 @@ def attach_3d_info(report, ui=None):
         if compiz_version:
             version = compiz_version.split(' ')[1]
             version = version[:3]
-            compiz_version_string = 'compiz-%s' % version
-            report['Tags'] += ' ' + compiz_version_string
+            if is_number(version):
+                compiz_version_string = 'compiz-%s' % version
+                report['Tags'] += ' ' + compiz_version_string
     elif is_process_running('kwin'):
         report['CompositorRunning'] = 'kwin'
     else:
@@ -490,7 +497,7 @@ def attach_input_device_info(report, ui=None):
     attach_command_output(report, ['gconftool-2', '-R', '/desktop/gnome/peripherals'], 'peripherals')
 
     # For keyboard bugs only
-    if not report.get('SourcePackage','Unknown') in keyboard_packages:
+    if is_xorg_keyboard_package(report.get('SourcePackage','Unknown')):
         attach_command_output(report, ['setxkbmap', '-print'], 'setxkbmap')
         attach_command_output(report, ['xkbcomp', ':0', '-w0', '-'], 'xkbcomp')
         attach_command_output(report, ['locale'], 'locale')
@@ -507,7 +514,7 @@ def attach_nvidia_info(report, ui=None):
     # Attach information for upstreaming nvidia binary bugs
     if nonfree_graphics_module() != 'nvidia':
         return
-    
+
     report['version.nvidia-graphics-drivers'] = package_versions("nvidia-graphics-drivers")
 
     for logfile in glob.glob('/proc/driver/nvidia/*'):
@@ -518,10 +525,13 @@ def attach_nvidia_info(report, ui=None):
         if os.path.basename(logfile) != 'README':
             attach_file(report, logfile)
 
-    if os.path.lexists('/usr/lib/nvidia-current/bin/nvidia-bug-report.sh'):
-        if retval(['/usr/lib/nvidia-current/bin/nvidia-bug-report.sh']) == 0:
+    if os.path.exists('/usr/bin/nvidia-bug-report.sh'):
+        if ui.yesno("Would you like to generate and attach an NVIDIA bug reporting file? (~/nvidia-bug-report.log.gz)") == True:
+            attach_root_command_outputs(report, {
+                    'NvidiaBugReportLog': 'nvidia-bug-report.sh',
+                })
             attach_file_if_exists(report, os.path.expanduser('~/nvidia-bug-report.log.gz'),
-                                      'NvidiaBugReportLog')
+                                  'nvidia-bug-report.log.gz')
 
     if os.environ.get('DISPLAY'):
         # Attach output of nvidia-settings --query if we've got a display
@@ -561,7 +571,7 @@ def attach_gpu_hang_info(report, ui):
         return
     if not 'freeze' in report['Tags']:
         return
-    
+
     if not ui.yesno("Apport has detected a possible GPU hang.  Did your system recently lock up and/or require a hard reboot?"):
         # If user isn't experiencing freeze symptoms, the remaining questions aren't relevant
         report['Tags'] += ' false-gpu-hang'
@@ -586,7 +596,7 @@ def attach_gpu_hang_info(report, ui):
     # Don't ask more questions if bug is infrequent
     if response < 3:
         return
-    
+
     text = "When did you first start experiencing these lockups?"
     choices = [
         "I don't know",
@@ -601,7 +611,7 @@ def attach_gpu_hang_info(report, ui):
     if response == None:
         raise StopIteration
     report['GpuHangStarted'] = choices[response[0]]
-        
+
     text = "Are you able to reproduce the lockup at will?"
     choices = [
         "I don't know",
@@ -618,15 +628,16 @@ def attach_debugging_interest_level(report, ui):
     if not ui:
         return
 
-    if (report.get('SourcePackage','Unknown') in core_x_packages or 
+    if (report.get('SourcePackage','Unknown') in core_x_packages or
         report.get('SourcePackage','Unknown') in video_packages):
         text = "Would you be willing to do additional debugging work?"
         choices = [
             "I don't know",
             "No",
-            "I just need an easy workaround",
+            "I just need to know a workaround",
             "Yes, if not too technical",
-            "Yes, whatever it takes to get this fixed in Ubuntu",
+            "Yes",
+            "Yes, including running git bisection searches",
             ]
         response = ui.choice(text, choices)
         if response == None:
@@ -652,9 +663,14 @@ def add_info(report, ui):
     attach_dist_upgrade_status(report, ui)
     debug("attach_hardware")
     attach_hardware(report)
+    debug("attach_xorg_info")
+    attach_xorg_info(report, ui)
 
     pkg = report.get('SourcePackage','Unknown')
-    if not is_xorg_input_package(pkg):
+    if is_xorg_input_package(pkg):
+        debug("attach_input_device_info")
+        attach_input_device_info(report, ui)
+    else:
         debug("attach_graphic_card_pci_info")
         attach_graphic_card_pci_info(report, ui)
         debug("attach_dkms_info")
@@ -668,10 +684,6 @@ def add_info(report, ui):
         debug("attach_3d_info")
         attach_3d_info(report, ui)
 
-    if not is_xorg_video_package(pkg):
-        debug("attach_input_device_info")
-        attach_input_device_info(report, ui)
-
     debug("attach_debugging_interest_level")
     attach_debugging_interest_level(report, ui)
     return True
@@ -684,7 +696,7 @@ if __name__ == '__main__':
 
     report = {}
     if not add_info(report, None):
-        print "Unreportable bug"
+        print("Unreportable bug")
         sys.exit(1)
     for key in report:
-        print '[%s]\n%s' % (key, report[key])
+        print('[%s]\n%s' % (key, report[key]))
